@@ -1,21 +1,31 @@
 # coding=utf-8
+# Copyright 2018 Sascha Schirra
 #
-# Copyright 2016 Sascha Schirra
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# This file is part of filebytes.
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer.
 #
-# filebytes is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
 #
-# filebytes software is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# 3. Neither the name of the copyright holder nor the names of its contributors
+# may be used to endorse or promote products derived from this software without
+# specific prior written permission.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" A ND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 
 from .enum import Enum
 from .binary import *
@@ -39,6 +49,7 @@ class IMAGE_FILE_MACHINE(Enum):
     POWERPCFP = 0x1f1
     THUMB = 0x1c2
     WCEMIPSV2 = 0x169
+    ARM64 = 0xaa64    
 
 class IMAGE_SCN(Enum):
     TYPE_NO_PAD = 0x00000008
@@ -510,6 +521,8 @@ class PE(Binary):
         offset = optional_header_offset + imageNtHeaders.header.FileHeader.SizeOfOptionalHeader  # start reading behind the dos- and ntheaders
 
         image_section_header_size = sizeof(IMAGE_SECTION_HEADER)
+        IMAGE_SIZEOF_SYMBOL = 18
+        strtable_offset = imageNtHeaders.header.FileHeader.PointerToSymbolTable + IMAGE_SIZEOF_SYMBOL * imageNtHeaders.header.FileHeader.NumberOfSymbols
 
         for sectionNo in range(imageNtHeaders.header.FileHeader.NumberOfSections):
             ishdr = IMAGE_SECTION_HEADER.from_buffer(data, offset)
@@ -522,7 +535,16 @@ class PE(Binary):
                 raw = (c_ubyte * size).from_buffer(data, ishdr.PointerToRawData)
                 bytes_ = bytearray(raw)
 
-            sections.append(SectionData(header=ishdr, name=ishdr.Name.decode('ASCII', errors='ignore'), bytes=bytes_, raw=raw))
+            secname = ishdr.Name.decode('ASCII', errors='ignore')
+            if secname.startswith('/'):
+                name_offset = int(secname[1:]) + strtable_offset
+                s = bytearray()
+                while self._bytes[name_offset] != 0:
+                    s.append(self._bytes[name_offset])
+                    name_offset += 1
+                secname = bytes(s).decode('ASCII', errors='ignore')
+
+            sections.append(SectionData(header=ishdr, name=secname, bytes=bytes_, raw=raw))
 
             offset += image_section_header_size
 
